@@ -4,6 +4,7 @@ from __future__ import absolute_import, print_function
 
 import logging
 from datetime import datetime
+from collections import OrderedDict
 
 from .base import MetaPubObject
 from .exceptions import MetaPubError
@@ -52,10 +53,15 @@ class PubMedArticle(MetaPubObject):
             super(PubMedArticle, self).__init__(xmlstr, 'PubmedBookArticle', args, kwargs)
             self.pubmed_type = 'book'
             self._root = 'BookDocument'
-        else:
+        elif xmlstr.find('<PubmedArticle>') > -1:
             super(PubMedArticle, self).__init__(xmlstr, 'PubmedArticle', args, kwargs)
             self.pubmed_type = 'article'
             self._root = 'MedlineCitation'
+        else:
+            # assume MedlineCitation is the root XML element
+            super(PubMedArticle, self).__init__(xmlstr, None, args, kwargs)
+            self.pubmed_type = 'article'
+            self._root = '.'
 
         pmt = self.pubmed_type
         
@@ -80,6 +86,18 @@ class PubMedArticle(MetaPubObject):
         self.pmc = None if pmt=='book' else self._get_pmc()
         self.issn = None if pmt=='book' else self._get_issn()
 
+        #MeSH headings ('article' only)
+        self.mesh = self._get_mesh_headings()
+
+        #Chemical associations ('article' only)
+        self.chemicals = self._get_chemicals()
+
+        #Grant information (?? 'article' only ??)
+        self.grants = self._get_grantlist()
+
+        #Publication Types (?? 'article' only ??)
+        self.publication_types = self._get_publication_types()
+
         # 'book' only:
         self.book_accession_id = None if pmt=='article' else self._get_bookaccession_id()
         self.book_title = None if pmt=='article' else self._get_book_title()
@@ -173,14 +191,17 @@ class PubMedArticle(MetaPubObject):
         return self._get('BookDocument/Book/Medium')
 
     def _get_book_contribution_date(self):
-        return self._construct_datetime(self.content.find('BookDocument/ContributionDate'))
+        contribution_date_element = self.content.find('BookDocument/ContributionDate')
+        if contribution_date_element is not None:
+            return self._construct_datetime(self.content.find('BookDocument/ContributionDate'))
+        return None
 
     def _get_book_date_revised(self):
         return self._construct_datetime(self.content.find('BookDocument/DateRevised'))
 
     def _get_book_synonyms(self):
         syn_list = self.content.find('BookDocument/ItemList')
-        if syn_list.get('ListType')=='Synonyms':
+        if syn_list is not None and syn_list.get('ListType')=='Synonyms':
             return [item.text for item in self.content.findall('BookDocument/ItemList/Item')]
         else:
             return []
@@ -196,7 +217,9 @@ class PubMedArticle(MetaPubObject):
         return self._get('PubmedBookData/PublicationStatus')
 
     def _get_book_year(self):
-        return self.book_contribution_date.year
+        if self.book_contribution_date:
+            return self.book_contribution_date.year
+        return None
 
     def _get_pmid(self):
         return self._get(self._root+'/PMID')
@@ -245,7 +268,6 @@ class PubMedArticle(MetaPubObject):
         if j is None:
             # e.g., http://www.ncbi.nlm.nih.gov/pubmed?term=21242195
             j = self._get(self._root+'/Article/Journal/Title')
-        assert j is not None
         return j
 
     def _get_pages(self):
@@ -305,8 +327,10 @@ class PubMedArticle(MetaPubObject):
         y = self._get(self._root+'/Article/Journal/JournalIssue/PubDate/Year')
         if y is None:
             # case applicable for pmid:9887384 (at least)
-            y = self._get(self._root+'/Article/Journal/JournalIssue/PubDate/MedlineDate')[0:4]
-        assert y is not None
+            try:
+                y = self._get(self._root+'/Article/Journal/JournalIssue/PubDate/MedlineDate')[0:4]
+            except TypeError:
+                pass
         return y
 
     def _get_doi(self):
@@ -323,7 +347,56 @@ class PubMedArticle(MetaPubObject):
 
     def _get_issn(self):
         return self._get(self._root+'/Article/Journal/ISSN')
+
+    def _get_mesh_headings(self):
+        if self.pubmed_type=='book':
+            return None
+
+        meshtags = self.content.findall('MedlineCitation/MeshHeadingList/MeshHeading')
+        outd = { }
+        for mesh in meshtags:
+            descript = mesh.find('DescriptorName')  # should always be present
+            qual = mesh.find('QualifierName')       # may not be present
+
+            DUI = descript.get('UI')
+            outd[DUI] = { 
+                    'descriptor_name': descript.text,
+                    'major_topic': True if descript.get('MajorTopicYN')=='Y' else False,
+                    'qualifier_name': None if qual is None else qual.text,
+                    'qualifier_ui': None if qual is None else qual.get('UI'),
+                }
+        return outd
+
+    def _get_chemicals(self):
+        if self.pubmed_type=='book':
+            return None
+
+        outd = {}
+        chemicals = self.content.findall('MedlineCitation/ChemicalList/Chemical')
+        for chem in chemicals:
+            substance = chem.find('NameOfSubstance')
+            regnum = chem.find('RegistryNumber').text  # very often this is '0'
+            outd[substance.get('UI')] = { 
+                    'substance_name': substance.text,
+                    'registry_number': regnum
+                }
+        return outd
+
+    def _get_publication_types(self):
+        outd = {}
+        pubtypes = self.content.findall('MedlineCitation/Article/PublicationTypeList/PublicationType')
+        for pt in pubtypes:
+            outd[pt.get('UI')] = pt.text
+        return outd
     
+    def _get_grantlist(self):
+        outl = []
+        grants = self.content.findall('MedlineCitation/GrantList')
+        for gr in grants:
+            outl.append( {'agency': gr.get('Agency', None), 'country': gr.get('Country', None)} )
+        return outl
+        
+
     def __str__(self):
         if self.pubmed_type == 'article':
             return( '%s (%s. %s, %s:%s)' % (
@@ -353,3 +426,18 @@ def _au_to_last_fm(au):
     raise MetaPubError("Author structure not recognized")
 
 
+def square_voliss_data_for_pma(pma):
+    '''takes a PubMedArticle object, returns same object with corrected volume/issue 
+    information (if needed)'''
+    if pma.volume != None and pma.issue is None:
+        # try to get a number out of the parts that came after the first number.
+        volparts = re_numbers.findall(pma.volume)
+        if len(volparts) > 1:
+            pma.volume = volparts[0]
+            # take a guess. best we can do. this often works (e.g. Brain journal)
+            pma.issue = volparts[1]
+    if pma.issue and pma.volume:
+        if pma.issue.find('Pt') > -1:
+            pma.issue = re_numbers.findall(pma.issue)[0]
+    return pma
+
